@@ -59,6 +59,14 @@ G_BEGIN_DECLS
  *  useful to know. As a rule of thumb a pipeline that is iterating as expected
  *  should never output anzthing else but LOG messages.
  *  Examples for this are referencing/dereferencing of objects or cothread switches.
+ * @GST_LEVEL_FIXME: Fixme messages are messages that indicate that something
+ *  in the executed code path is not fully implemented or handled yet. Note
+ *  that this does not replace proper error handling in any way, the purpose
+ *  of this message is to make it easier to spot incomplete/unfinished pieces
+ *  of code when reading the debug log. (Since: 0.10.23)
+ * @GST_LEVEL_MEMDUMP: memory dump messages are used to log (small) chunks of
+ *  data as memory dumps in the log. They will be displayed as hexdump with
+ *  ASCII characters. (Since: 0.10.23)
  * @GST_LEVEL_COUNT: The number of defined debugging levels.
  *
  * The level defines the importance of a debugging message. The more important a
@@ -71,6 +79,9 @@ typedef enum {
   GST_LEVEL_INFO,
   GST_LEVEL_DEBUG,
   GST_LEVEL_LOG,
+  GST_LEVEL_FIXME = 6,
+  /* add more */
+  GST_LEVEL_MEMDUMP = 9,
   /* add more */
   GST_LEVEL_COUNT
 } GstDebugLevel;
@@ -209,7 +220,7 @@ struct _GstDebugCategory {
 #ifndef GST_FUNCTION
 #if defined (__GNUC__) || (defined (_MSC_VER) && _MSC_VER >= 1300)
 #  define GST_FUNCTION     ((const char*) (__FUNCTION__))
-#elif defined (G_HAVE_ISO_VARARGS)
+#elif defined (__STDC__) && defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
 #  define GST_FUNCTION     ((const char*) (__func__))
 #else
 #  define GST_FUNCTION     ((const char*) ("???"))
@@ -339,7 +350,8 @@ void            gst_debug_unset_threshold_for_name  (const gchar * name);
  * This macro expands to nothing if debugging is disabled.
  */
 #define GST_DEBUG_CATEGORY_STATIC(cat) static GstDebugCategory *cat = NULL
-/* do not use this function, use the macros below */
+
+/* do not use this function, use the GST_DEBUG_CATEGORY_INIT macro below */
 GstDebugCategory *_gst_debug_category_new (const gchar * name,
                                            guint         color,
                                            const gchar * description);
@@ -397,7 +409,45 @@ G_CONST_RETURN gchar *
 GSList *
         gst_debug_get_all_categories	(void);
 
-gchar *	gst_debug_construct_term_color	(guint colorinfo);
+/* do not use this function, use the GST_DEBUG_CATEGORY_GET macro below */
+GstDebugCategory *_gst_debug_get_category (const gchar *name);
+
+/**
+ * GST_DEBUG_CATEGORY_GET:
+ * @cat: the category to initialize.
+ * @name: log category name
+ *
+ * Lookup an exiting #GstDebugCategory by its @name and sets @cat. If category
+ * is not found, but %GST_CAT_DEFAULT is defined, that is assigned to @cat.
+ * Otherwise cat will be NULL.
+ *
+ * |[
+ * GST_DEBUG_CATEGORY_STATIC (gst_myplugin_debug);
+ * #define GST_CAT_DEFAULT gst_myplugin_debug
+ * GST_DEBUG_CATEGORY_STATIC (GST_CAT_PERFORMANCE);
+ * ...
+ * GST_DEBUG_CATEGORY_INIT (gst_myplugin_debug, "myplugin", 0, "nice element");
+ * GST_DEBUG_CATEGORY_GET (GST_CAT_PERFORMANCE, "performance);
+ * ]|
+ *
+ * Since: 0.10.24
+ */
+#ifdef GST_CAT_DEFAULT
+#define GST_DEBUG_CATEGORY_GET(cat,name)  G_STMT_START{\
+  cat = _gst_debug_get_category (name);			\
+  if (!cat) {						\
+    cat = GST_CAT_DEFAULT;				\
+  }							\
+}G_STMT_END
+#else
+#define GST_DEBUG_CATEGORY_GET(cat,name)  G_STMT_START{\
+  cat = _gst_debug_get_category (name);			\
+}G_STMT_END
+#endif
+
+
+gchar * gst_debug_construct_term_color	(guint colorinfo);
+gint    gst_debug_construct_win_color (guint colorinfo);
 
 
 /**
@@ -463,6 +513,30 @@ GST_CAT_LEVEL_LOG (GstDebugCategory * cat, GstDebugLevel level,
 #endif
 #endif /* G_HAVE_ISO_VARARGS */
 
+/* private helper function */
+void _gst_debug_dump_mem (GstDebugCategory * cat, const gchar * file,
+    const gchar * func, gint line, GObject * obj, const gchar * msg,
+    const guint8 * data, guint length);
+
+/* This one doesn't have varargs in the macro, so it's different than all the
+ * other macros and hence in a separate block right here. Docs chunks are
+ * with the other doc chunks below though. */
+#define __GST_CAT_MEMDUMP_LOG(cat,object,msg,data,length) G_STMT_START{       \
+  if (G_UNLIKELY (GST_LEVEL_MEMDUMP <= __gst_debug_min)) {                    \
+    _gst_debug_dump_mem ((cat), __FILE__, GST_FUNCTION, __LINE__,             \
+        (GObject *) (object), (msg), (data), (length));                       \
+  }                                                                           \
+}G_STMT_END
+
+#define GST_CAT_MEMDUMP_OBJECT(cat,obj,msg,data,length)  \
+    __GST_CAT_MEMDUMP_LOG(cat,obj,msg,data,length)
+#define GST_CAT_MEMDUMP(cat,msg,data,length)             \
+    __GST_CAT_MEMDUMP_LOG(cat,NULL,msg,data,length)
+#define GST_MEMDUMP_OBJECT(obj,msg,data,length)          \
+    __GST_CAT_MEMDUMP_LOG(GST_CAT_DEFAULT,obj,msg,data,length)
+#define GST_MEMDUMP(msg,data,length)                     \
+    __GST_CAT_MEMDUMP_LOG(GST_CAT_DEFAULT,NULL,msg,data,length)
+
 /**
  * GST_CAT_ERROR_OBJECT:
  * @cat: category to use
@@ -504,6 +578,29 @@ GST_CAT_LEVEL_LOG (GstDebugCategory * cat, GstDebugLevel level,
  *
  * Output an logging message belonging to the given object in the given category.
  */
+/**
+ * GST_CAT_FIXME_OBJECT:
+ * @cat: category to use
+ * @obj: the #GObject the message belongs to
+ * @...: printf-style message to output
+ *
+ * Output a fixme message belonging to the given object in the given category.
+ *
+ * Since: 0.10.23
+ */
+/**
+ * GST_CAT_MEMDUMP_OBJECT:
+ * @cat: category to use
+ * @obj: the #GObject the message belongs to
+ * @msg: message string to log with the data
+ * @data: pointer to the data to output
+ * @length: length of the data to output
+ *
+ * Output a hexdump of @data relating to the given object in the given
+ * category.
+ *
+ * Since: 0.10.23
+ */
 
 
 /**
@@ -541,6 +638,26 @@ GST_CAT_LEVEL_LOG (GstDebugCategory * cat, GstDebugLevel level,
  *
  * Output an logging message in the given category.
  */
+/**
+ * GST_CAT_FIXME:
+ * @cat: category to use
+ * @...: printf-style message to output
+ *
+ * Output an fixme message in the given category.
+ *
+ * Since: 0.10.23
+ */
+/**
+ * GST_CAT_MEMDUMP:
+ * @cat: category to use
+ * @msg: message string to log with the data
+ * @data: pointer to the data to output
+ * @length: length of the data to output
+ *
+ * Output a hexdump of @data in the given category.
+ *
+ * Since: 0.10.23
+ */
 
 
 /**
@@ -580,6 +697,26 @@ GST_CAT_LEVEL_LOG (GstDebugCategory * cat, GstDebugLevel level,
  *
  * Output a logging message belonging to the given object in the default category.
  */
+/**
+ * GST_FIXME_OBJECT:
+ * @obj: the #GObject the message belongs to
+ * @...: printf-style message to output
+ *
+ * Output a logging message belonging to the given object in the default category.
+ *
+ * Since: 0.10.23
+ */
+/**
+ * GST_MEMDUMP_OBJECT:
+ * @obj: the #GObject the message belongs to
+ * @msg: message string to log with the data
+ * @data: pointer to the data to output
+ * @length: length of the data to output
+ *
+ * Output a logging message belonging to the given object in the default category.
+ *
+ * Since: 0.10.23
+ */
 
 
 /**
@@ -612,6 +749,24 @@ GST_CAT_LEVEL_LOG (GstDebugCategory * cat, GstDebugLevel level,
  *
  * Output a logging message in the default category.
  */
+/**
+ * GST_FIXME:
+ * @...: printf-style message to output
+ *
+ * Output a fixme message in the default category.
+ *
+ * Since: 0.10.23
+ */
+/**
+ * GST_MEMDUMP:
+ * @msg: message string to log with the data
+ * @data: pointer to the data to output
+ * @length: length of the data to output
+ *
+ * Output a hexdump of @data.
+ *
+ * Since: 0.10.23
+ */
 
 #ifdef G_HAVE_ISO_VARARGS
 
@@ -620,24 +775,28 @@ GST_CAT_LEVEL_LOG (GstDebugCategory * cat, GstDebugLevel level,
 #define GST_CAT_INFO_OBJECT(cat,obj,...)	GST_CAT_LEVEL_LOG (cat, GST_LEVEL_INFO,    obj,  __VA_ARGS__)
 #define GST_CAT_DEBUG_OBJECT(cat,obj,...)	GST_CAT_LEVEL_LOG (cat, GST_LEVEL_DEBUG,   obj,  __VA_ARGS__)
 #define GST_CAT_LOG_OBJECT(cat,obj,...)		GST_CAT_LEVEL_LOG (cat, GST_LEVEL_LOG,     obj,  __VA_ARGS__)
+#define GST_CAT_FIXME_OBJECT(cat,obj,...)	GST_CAT_LEVEL_LOG (cat, GST_LEVEL_FIXME,   obj,  __VA_ARGS__)
 
 #define GST_CAT_ERROR(cat,...)			GST_CAT_LEVEL_LOG (cat, GST_LEVEL_ERROR,   NULL, __VA_ARGS__)
 #define GST_CAT_WARNING(cat,...)		GST_CAT_LEVEL_LOG (cat, GST_LEVEL_WARNING, NULL, __VA_ARGS__)
 #define GST_CAT_INFO(cat,...)			GST_CAT_LEVEL_LOG (cat, GST_LEVEL_INFO,    NULL, __VA_ARGS__)
 #define GST_CAT_DEBUG(cat,...)			GST_CAT_LEVEL_LOG (cat, GST_LEVEL_DEBUG,   NULL, __VA_ARGS__)
 #define GST_CAT_LOG(cat,...)			GST_CAT_LEVEL_LOG (cat, GST_LEVEL_LOG,     NULL, __VA_ARGS__)
+#define GST_CAT_FIXME(cat,...)			GST_CAT_LEVEL_LOG (cat, GST_LEVEL_FIXME,   NULL, __VA_ARGS__)
 
 #define GST_ERROR_OBJECT(obj,...)	GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_ERROR,   obj,  __VA_ARGS__)
 #define GST_WARNING_OBJECT(obj,...)	GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_WARNING, obj,  __VA_ARGS__)
 #define GST_INFO_OBJECT(obj,...)	GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_INFO,    obj,  __VA_ARGS__)
 #define GST_DEBUG_OBJECT(obj,...)	GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_DEBUG,   obj,  __VA_ARGS__)
 #define GST_LOG_OBJECT(obj,...)		GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_LOG,     obj,  __VA_ARGS__)
+#define GST_FIXME_OBJECT(obj,...)	GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_FIXME,   obj,  __VA_ARGS__)
 
 #define GST_ERROR(...)			GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_ERROR,   NULL, __VA_ARGS__)
 #define GST_WARNING(...)		GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_WARNING, NULL, __VA_ARGS__)
 #define GST_INFO(...)			GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_INFO,    NULL, __VA_ARGS__)
 #define GST_DEBUG(...)			GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_DEBUG,   NULL, __VA_ARGS__)
 #define GST_LOG(...)			GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_LOG,     NULL, __VA_ARGS__)
+#define GST_FIXME(...)			GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_FIXME,   NULL, __VA_ARGS__)
 
 #else
 #ifdef G_HAVE_GNUC_VARARGS
@@ -647,24 +806,28 @@ GST_CAT_LEVEL_LOG (GstDebugCategory * cat, GstDebugLevel level,
 #define GST_CAT_INFO_OBJECT(cat,obj,args...)	GST_CAT_LEVEL_LOG (cat, GST_LEVEL_INFO,    obj,  ##args )
 #define GST_CAT_DEBUG_OBJECT(cat,obj,args...)	GST_CAT_LEVEL_LOG (cat, GST_LEVEL_DEBUG,   obj,  ##args )
 #define GST_CAT_LOG_OBJECT(cat,obj,args...)	GST_CAT_LEVEL_LOG (cat, GST_LEVEL_LOG,     obj,  ##args )
+#define GST_CAT_FIXME_OBJECT(cat,obj,args...)	GST_CAT_LEVEL_LOG (cat, GST_LEVEL_FIXME,   obj,  ##args )
 
 #define GST_CAT_ERROR(cat,args...)		GST_CAT_LEVEL_LOG (cat, GST_LEVEL_ERROR,   NULL, ##args )
 #define GST_CAT_WARNING(cat,args...)		GST_CAT_LEVEL_LOG (cat, GST_LEVEL_WARNING, NULL, ##args )
 #define GST_CAT_INFO(cat,args...)		GST_CAT_LEVEL_LOG (cat, GST_LEVEL_INFO,    NULL, ##args )
 #define GST_CAT_DEBUG(cat,args...)		GST_CAT_LEVEL_LOG (cat, GST_LEVEL_DEBUG,   NULL, ##args )
 #define GST_CAT_LOG(cat,args...)		GST_CAT_LEVEL_LOG (cat, GST_LEVEL_LOG,     NULL, ##args )
+#define GST_CAT_FIXME(cat,args...)		GST_CAT_LEVEL_LOG (cat, GST_LEVEL_FIXME,   NULL, ##args )
 
 #define GST_ERROR_OBJECT(obj,args...)	GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_ERROR,   obj,  ##args )
 #define GST_WARNING_OBJECT(obj,args...)	GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_WARNING, obj,  ##args )
 #define GST_INFO_OBJECT(obj,args...)	GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_INFO,    obj,  ##args )
 #define GST_DEBUG_OBJECT(obj,args...)	GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_DEBUG,   obj,  ##args )
 #define GST_LOG_OBJECT(obj,args...)	GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_LOG,     obj,  ##args )
+#define GST_FIXME_OBJECT(obj,args...)	GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_FIXME,   obj,  ##args )
 
 #define GST_ERROR(args...)		GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_ERROR,   NULL, ##args )
 #define GST_WARNING(args...)		GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_WARNING, NULL, ##args )
 #define GST_INFO(args...)		GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_INFO,    NULL, ##args )
 #define GST_DEBUG(args...)		GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_DEBUG,   NULL, ##args )
 #define GST_LOG(args...)		GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_LOG,     NULL, ##args )
+#define GST_FIXME(args...)		GST_CAT_LEVEL_LOG (GST_CAT_DEFAULT, GST_LEVEL_FIXME,   NULL, ##args )
 
 #else
 /* no variadic macros, use inline */
@@ -723,6 +886,17 @@ GST_CAT_LOG_OBJECT (GstDebugCategory * cat, gpointer obj, const char *format,
   va_end (varargs);
 }
 
+static inline void
+GST_CAT_FIXME_OBJECT (GstDebugCategory * cat, gpointer obj, const char *format,
+    ...)
+{
+  va_list varargs;
+
+  va_start (varargs, format);
+  GST_CAT_LEVEL_LOG_valist (cat, GST_LEVEL_FIXME, obj, format, varargs);
+  va_end (varargs);
+}
+
 static inline void
 GST_CAT_ERROR (GstDebugCategory * cat, const char *format, ...)
 {
@@ -773,6 +947,16 @@ GST_CAT_LOG (GstDebugCategory * cat, const char *format, ...)
   va_end (varargs);
 }
 
+static inline void
+GST_CAT_FIXME (GstDebugCategory * cat, const char *format, ...)
+{
+  va_list varargs;
+
+  va_start (varargs, format);
+  GST_CAT_LEVEL_LOG_valist (cat, GST_LEVEL_FIXME, NULL, format, varargs);
+  va_end (varargs);
+}
+
 static inline void
 GST_ERROR_OBJECT (gpointer obj, const char *format, ...)
 {
@@ -828,6 +1012,17 @@ GST_LOG_OBJECT (gpointer obj, const char *format, ...)
   va_end (varargs);
 }
 
+static inline void
+GST_FIXME_OBJECT (gpointer obj, const char *format, ...)
+{
+  va_list varargs;
+
+  va_start (varargs, format);
+  GST_CAT_LEVEL_LOG_valist (GST_CAT_DEFAULT, GST_LEVEL_FIXME, obj, format,
+      varargs);
+  va_end (varargs);
+}
+
 static inline void
 GST_ERROR (const char *format, ...)
 {
@@ -882,6 +1077,17 @@ GST_LOG (const char *format, ...)
       format, varargs);
   va_end (varargs);
 }
+
+static inline void
+GST_FIXME (const char *format, ...)
+{
+  va_list varargs;
+
+  va_start (varargs, format);
+  GST_CAT_LEVEL_LOG_valist (GST_CAT_DEFAULT, GST_LEVEL_FIXME, NULL, format,
+      varargs);
+  va_end (varargs);
+}
 #endif
 #endif
 
@@ -921,107 +1127,123 @@ G_CONST_RETURN gchar *
 #else /* GST_DISABLE_GST_DEBUG */
 
 
+#ifndef GST_INFO_C
+
 #if defined(__GNUC__) && __GNUC__ >= 3
 #  pragma GCC poison gst_debug_log
 #  pragma GCC poison gst_debug_log_valist
 #  pragma GCC poison _gst_debug_category_new
 #endif
 
-#define _gst_debug_init()				/* NOP */
+#define __gst_debug_min GST_LEVEL_NONE
 
-#define gst_debug_set_default_threshold(level)		/* NOP */
+#define _gst_debug_init()				G_STMT_START{ }G_STMT_END
+
+#define gst_debug_set_default_threshold(level)		G_STMT_START{ }G_STMT_END
 #define gst_debug_get_default_threshold()		(GST_LEVEL_NONE)
 
 #define gst_debug_level_get_name(level)				("NONE")
-#define gst_debug_message_get(message)  			("NONE")
-#define gst_debug_add_log_function(func,data)		/* NOP */
+#define gst_debug_message_get(message)  			("")
+#define gst_debug_add_log_function(func,data)		G_STMT_START{ }G_STMT_END
 guint gst_debug_remove_log_function (GstLogFunction func);
 guint gst_debug_remove_log_function_by_data (gpointer data);
-#define gst_debug_set_active(active)			/* NOP */
+#define gst_debug_set_active(active)			G_STMT_START{ }G_STMT_END
 #define gst_debug_is_active()				(FALSE)
-#define gst_debug_set_colored(colored)			/* NOP */
+#define gst_debug_set_colored(colored)			G_STMT_START{ }G_STMT_END
 #define gst_debug_is_colored()				(FALSE)
-#define gst_debug_set_default_threshold(level)		/* NOP */
+#define gst_debug_set_default_threshold(level)		G_STMT_START{ }G_STMT_END
 #define gst_debug_get_default_threshold()		(GST_LEVEL_NONE)
-#define gst_debug_set_threshold_for_name(name,level)	/* NOP */
-#define gst_debug_unset_threshold_for_name(name)	/* NOP */
+#define gst_debug_set_threshold_for_name(name,level)	G_STMT_START{ }G_STMT_END
+#define gst_debug_unset_threshold_for_name(name)	G_STMT_START{ }G_STMT_END
 
-#define GST_DEBUG_CATEGORY(var)				/* NOP */
-#define GST_DEBUG_CATEGORY_EXTERN(var)			/* NOP */
+#define GST_DEBUG_CATEGORY(var)				void _gst_dummy_declaration (void)
+#define GST_DEBUG_CATEGORY_EXTERN(var)			void _gst_dummy_declaration (void)
 #if !defined(G_HAVE_GNUC_VARARGS) && !defined(G_HAVE_ISO_VARARGS)
 #define GST_DEBUG_CATEGORY_STATIC(var)			static GstDebugCategory *var = NULL
 #else
-#define GST_DEBUG_CATEGORY_STATIC(var)			/* NOP */
+#define GST_DEBUG_CATEGORY_STATIC(var)			void _gst_dummy_declaration (void)
 #endif
-#define GST_DEBUG_CATEGORY_INIT(var,name,color,desc)	/* NOP */
-#define gst_debug_category_free(category)		/* NOP */
-#define gst_debug_category_set_threshold(category,level) /* NOP */
-#define gst_debug_category_reset_threshold(category)	/* NOP */
+#define GST_DEBUG_CATEGORY_INIT(var,name,color,desc)	G_STMT_START{ }G_STMT_END
+#define GST_DEBUG_CATEGORY_GET(var,name)		G_STMT_START{ }G_STMT_END
+#define gst_debug_category_free(category)		G_STMT_START{ }G_STMT_END
+#define gst_debug_category_set_threshold(category,level) G_STMT_START{ }G_STMT_END
+#define gst_debug_category_reset_threshold(category)	G_STMT_START{ }G_STMT_END
 #define gst_debug_category_get_threshold(category)	(GST_LEVEL_NONE)
 #define gst_debug_category_get_name(cat)		("")
 #define gst_debug_category_get_color(cat)		(0)
 #define gst_debug_category_get_description(cat)		("")
 #define gst_debug_get_all_categories()			(NULL)
 #define gst_debug_construct_term_color(colorinfo)	(g_strdup ("00"))
+#define gst_debug_construct_win_color(colorinfo)	(0)
+
+#endif /* !GST_INFO_C */
 
 #ifdef G_HAVE_ISO_VARARGS
 
-#define GST_CAT_LEVEL_LOG(cat,level,...)		/* NOP */
+#define GST_CAT_LEVEL_LOG(cat,level,...)		G_STMT_START{ }G_STMT_END
 
-#define GST_CAT_ERROR_OBJECT(...)			/* NOP */
-#define GST_CAT_WARNING_OBJECT(...)			/* NOP */
-#define GST_CAT_INFO_OBJECT(...)			/* NOP */
-#define GST_CAT_DEBUG_OBJECT(...)			/* NOP */
-#define GST_CAT_LOG_OBJECT(...)				/* NOP */
+#define GST_CAT_ERROR_OBJECT(...)			G_STMT_START{ }G_STMT_END
+#define GST_CAT_WARNING_OBJECT(...)			G_STMT_START{ }G_STMT_END
+#define GST_CAT_INFO_OBJECT(...)			G_STMT_START{ }G_STMT_END
+#define GST_CAT_DEBUG_OBJECT(...)			G_STMT_START{ }G_STMT_END
+#define GST_CAT_LOG_OBJECT(...)				G_STMT_START{ }G_STMT_END
+#define GST_CAT_FIXME_OBJECT(...)			G_STMT_START{ }G_STMT_END
 
-#define GST_CAT_ERROR(...)				/* NOP */
-#define GST_CAT_WARNING(...)				/* NOP */
-#define GST_CAT_INFO(...)				/* NOP */
-#define GST_CAT_DEBUG(...)				/* NOP */
-#define GST_CAT_LOG(...)				/* NOP */
+#define GST_CAT_ERROR(...)				G_STMT_START{ }G_STMT_END
+#define GST_CAT_WARNING(...)				G_STMT_START{ }G_STMT_END
+#define GST_CAT_INFO(...)				G_STMT_START{ }G_STMT_END
+#define GST_CAT_DEBUG(...)				G_STMT_START{ }G_STMT_END
+#define GST_CAT_LOG(...)				G_STMT_START{ }G_STMT_END
+#define GST_CAT_FIXME(...)				G_STMT_START{ }G_STMT_END
 
-#define GST_ERROR_OBJECT(...)				/* NOP */
-#define GST_WARNING_OBJECT(...)				/* NOP */
-#define GST_INFO_OBJECT(...)				/* NOP */
-#define GST_DEBUG_OBJECT(...)				/* NOP */
-#define GST_LOG_OBJECT(...)				/* NOP */
+#define GST_ERROR_OBJECT(...)				G_STMT_START{ }G_STMT_END
+#define GST_WARNING_OBJECT(...)				G_STMT_START{ }G_STMT_END
+#define GST_INFO_OBJECT(...)				G_STMT_START{ }G_STMT_END
+#define GST_DEBUG_OBJECT(...)				G_STMT_START{ }G_STMT_END
+#define GST_LOG_OBJECT(...)				G_STMT_START{ }G_STMT_END
+#define GST_FIXME_OBJECT(...)				G_STMT_START{ }G_STMT_END
 
-#define GST_ERROR(...)					/* NOP */
-#define GST_WARNING(...)				/* NOP */
-#define GST_INFO(...)					/* NOP */
-#define GST_DEBUG(...)					/* NOP */
-#define GST_LOG(...)					/* NOP */
+#define GST_ERROR(...)					G_STMT_START{ }G_STMT_END
+#define GST_WARNING(...)				G_STMT_START{ }G_STMT_END
+#define GST_INFO(...)					G_STMT_START{ }G_STMT_END
+#define GST_DEBUG(...)					G_STMT_START{ }G_STMT_END
+#define GST_LOG(...)					G_STMT_START{ }G_STMT_END
+#define GST_FIXME(...)					G_STMT_START{ }G_STMT_END
 
-#else
+#else /* !G_HAVE_ISO_VARARGS */
 #ifdef G_HAVE_GNUC_VARARGS
 
-#define GST_CAT_LEVEL_LOG(cat,level,args...)		/* NOP */
+#define GST_CAT_LEVEL_LOG(cat,level,args...)		G_STMT_START{ }G_STMT_END
 
-#define GST_CAT_ERROR_OBJECT(args...)			/* NOP */
-#define GST_CAT_WARNING_OBJECT(args...)			/* NOP */
-#define GST_CAT_INFO_OBJECT(args...)			/* NOP */
-#define GST_CAT_DEBUG_OBJECT(args...)			/* NOP */
-#define GST_CAT_LOG_OBJECT(args...)			/* NOP */
+#define GST_CAT_ERROR_OBJECT(args...)			G_STMT_START{ }G_STMT_END
+#define GST_CAT_WARNING_OBJECT(args...)			G_STMT_START{ }G_STMT_END
+#define GST_CAT_INFO_OBJECT(args...)			G_STMT_START{ }G_STMT_END
+#define GST_CAT_DEBUG_OBJECT(args...)			G_STMT_START{ }G_STMT_END
+#define GST_CAT_LOG_OBJECT(args...)			G_STMT_START{ }G_STMT_END
+#define GST_CAT_FIXME_OBJECT(args...)			G_STMT_START{ }G_STMT_END
 
-#define GST_CAT_ERROR(args...)				/* NOP */
-#define GST_CAT_WARNING(args...)			/* NOP */
-#define GST_CAT_INFO(args...)				/* NOP */
-#define GST_CAT_DEBUG(args...)				/* NOP */
-#define GST_CAT_LOG(args...)				/* NOP */
+#define GST_CAT_ERROR(args...)				G_STMT_START{ }G_STMT_END
+#define GST_CAT_WARNING(args...)			G_STMT_START{ }G_STMT_END
+#define GST_CAT_INFO(args...)				G_STMT_START{ }G_STMT_END
+#define GST_CAT_DEBUG(args...)				G_STMT_START{ }G_STMT_END
+#define GST_CAT_LOG(args...)				G_STMT_START{ }G_STMT_END
+#define GST_CAT_FIXME(args...)				G_STMT_START{ }G_STMT_END
 
-#define GST_ERROR_OBJECT(args...)			/* NOP */
-#define GST_WARNING_OBJECT(args...)			/* NOP */
-#define GST_INFO_OBJECT(args...)			/* NOP */
-#define GST_DEBUG_OBJECT(args...)			/* NOP */
-#define GST_LOG_OBJECT(args...)				/* NOP */
+#define GST_ERROR_OBJECT(args...)			G_STMT_START{ }G_STMT_END
+#define GST_WARNING_OBJECT(args...)			G_STMT_START{ }G_STMT_END
+#define GST_INFO_OBJECT(args...)			G_STMT_START{ }G_STMT_END
+#define GST_DEBUG_OBJECT(args...)			G_STMT_START{ }G_STMT_END
+#define GST_LOG_OBJECT(args...)				G_STMT_START{ }G_STMT_END
+#define GST_FIXME_OBJECT(args...)			G_STMT_START{ }G_STMT_END
 
-#define GST_ERROR(args...)				/* NOP */
-#define GST_WARNING(args...)				/* NOP */
-#define GST_INFO(args...)				/* NOP */
-#define GST_DEBUG(args...)				/* NOP */
-#define GST_LOG(args...)				/* NOP */
+#define GST_ERROR(args...)				G_STMT_START{ }G_STMT_END
+#define GST_WARNING(args...)				G_STMT_START{ }G_STMT_END
+#define GST_INFO(args...)				G_STMT_START{ }G_STMT_END
+#define GST_DEBUG(args...)				G_STMT_START{ }G_STMT_END
+#define GST_LOG(args...)				G_STMT_START{ }G_STMT_END
+#define GST_FIXME(args...)				G_STMT_START{ }G_STMT_END
 
-#else
+#else /* !G_HAVE_GNUC_VARARGS */
 static inline void
 GST_CAT_LEVEL_LOG_valist (GstDebugCategory * cat,
     GstDebugLevel level, gpointer object, const char *format, va_list varargs)
@@ -1058,6 +1280,12 @@ GST_CAT_LOG_OBJECT (GstDebugCategory * cat, gpointer obj, const char *format,
 {
 }
 
+static inline void
+GST_CAT_FIXME_OBJECT (GstDebugCategory * cat, gpointer obj, const char *format,
+    ...)
+{
+}
+
 static inline void
 GST_CAT_ERROR (GstDebugCategory * cat, const char *format, ...)
 {
@@ -1083,6 +1311,11 @@ GST_CAT_LOG (GstDebugCategory * cat, const char *format, ...)
 {
 }
 
+static inline void
+GST_CAT_FIXME (GstDebugCategory * cat, const char *format, ...)
+{
+}
+
 static inline void
 GST_ERROR_OBJECT (gpointer obj, const char *format, ...)
 {
@@ -1108,6 +1341,11 @@ GST_LOG_OBJECT (gpointer obj, const char *format, ...)
 {
 }
 
+static inline void
+GST_FIXME_OBJECT (gpointer obj, const char *format, ...)
+{
+}
+
 static inline void
 GST_ERROR (const char *format, ...)
 {
@@ -1132,12 +1370,23 @@ static inline void
 GST_LOG (const char *format, ...)
 {
 }
-#endif
-#endif
+
+static inline void
+GST_FIXME (const char *format, ...)
+{
+}
+
+#endif /* G_HAVE_GNUC_VARARGS */
+#endif /* G_HAVE_ISO_VARARGS */
 
 #define GST_DEBUG_FUNCPTR(ptr) (ptr)
 #define GST_DEBUG_FUNCPTR_NAME(ptr) (g_strdup_printf ("%p", ptr))
 
+#define GST_CAT_MEMDUMP_OBJECT(cat,obj,msg,data,length) G_STMT_START{ }G_STMT_END
+#define GST_CAT_MEMDUMP(cat,msg,data,length)            G_STMT_START{ }G_STMT_END
+#define GST_MEMDUMP_OBJECT(obj,msg,data,length)         G_STMT_START{ }G_STMT_END
+#define GST_MEMDUMP(msg,data,length)                    G_STMT_START{ }G_STMT_END
+
 #endif /* GST_DISABLE_GST_DEBUG */
 
 
