@@ -49,7 +49,7 @@ G_BEGIN_DECLS
 
 /**
  * GstObjectFlags:
- * @GST_OBJECT_DISPOSING: the object is been destroyed, do use it anymore
+ * @GST_OBJECT_DISPOSING: the object is been destroyed, don't use it anymore
  * @GST_OBJECT_FLOATING:  the object has a floating reference count (e.g. its
  *  not assigned to a bin)
  * @GST_OBJECT_FLAG_LAST: subclasses can add additional flags starting from this flag
@@ -290,6 +290,7 @@ void            gst_object_default_deep_notify 	(GObject *object, GstObject *ori
 /* refcounting + life cycle */
 gpointer	gst_object_ref			(gpointer object);
 void		gst_object_unref		(gpointer object);
+void 		gst_object_ref_sink		(gpointer object);
 void 		gst_object_sink			(gpointer object);
 
 /* replace object pointer */
@@ -306,7 +307,7 @@ gboolean	gst_object_check_uniqueness	(GList *list, const gchar *name);
 GstXmlNodePtr   gst_object_save_thyself    (GstObject *object, GstXmlNodePtr parent);
 void            gst_object_restore_thyself (GstObject *object, GstXmlNodePtr self);
 #else
-#if defined _GNUC_ && _GNUC_ >= 3
+#if defined __GNUC__ && __GNUC__ >= 3
 #pragma GCC poison gst_object_save_thyself
 #pragma GCC poison gst_object_restore_thyself
 #endif
@@ -323,7 +324,7 @@ void        gst_class_signal_emit_by_name   (GstObject     * object,
                                              const gchar   * name,
                                              GstXmlNodePtr   self);
 #else
-#if defined _GNUC_ && _GNUC_ >= 3
+#if defined __GNUC__ && __GNUC__ >= 3
 #pragma GCC poison gst_class_signal_emit_by_name
 #endif
 #endif
