@@ -20,6 +20,10 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <gst/check/gstcheck.h>
 
 /*
@@ -38,12 +42,12 @@ struct _GstFakeObjectClass
   GstObjectClass parent_class;
 };
 
-GType _gst_fake_object_type = 0;
+static GType _gst_fake_object_type = 0;
 
 //static GstObjectClass *parent_class = NULL;
 //static guint gst_fake_object_signals[LAST_SIGNAL] = { 0 };
 
-GType
+static GType
 gst_fake_object_get_type (void)
 {
   if (!_gst_fake_object_type) {
@@ -75,7 +79,8 @@ GST_START_TEST (test_fail_abstract_new)
   fail_unless (object == NULL, "Created an instance of abstract GstObject");
 }
 
-GST_END_TEST
+GST_END_TEST;
+
 /* g_object_new on GstFakeObject should succeed */
 GST_START_TEST (test_fake_object_new)
 {
@@ -88,7 +93,8 @@ GST_START_TEST (test_fake_object_new)
   gst_object_unref (object);
 }
 
-GST_END_TEST
+GST_END_TEST;
+
 /* GstFakeObject name tests */
 GST_START_TEST (test_fake_object_name)
 {
@@ -129,9 +135,11 @@ GST_START_TEST (test_fake_object_name)
   gst_object_unref (object);
 }
 
-GST_END_TEST
+GST_END_TEST;
+
 /* thread function for threaded name change test */
-    gpointer thread_name_object (GstObject * object)
+static gpointer
+thread_name_object (GstObject * object)
 {
   gchar *thread_id = g_strdup_printf ("%p", g_thread_self ());
 
@@ -227,18 +235,19 @@ GST_START_TEST (test_fake_object_name_threaded_right)
   gst_object_unref (object);
 }
 
-GST_END_TEST
+GST_END_TEST;
 /*
  * main thread creates lots of objects
  * child threads sets default names on objects
  * then main thread checks uniqueness of object names
  */
-    GList * object_list = NULL;
-gint num_objects = 1000;
-gint num_threads = 5;
+
+static GList *object_list = NULL;
+static gint num_objects = 1000;
+static gint num_threads = 5;
 
 /* thread function for threaded default name change test */
-gpointer
+static gpointer
 thread_name_object_default (int *i)
 {
   int j;
@@ -333,7 +342,8 @@ GST_START_TEST (test_fake_object_name_threaded_unique)
   g_list_foreach (object_list, (GFunc) g_object_unref, NULL);
 }
 
-GST_END_TEST
+GST_END_TEST;
+
 /* parentage test on GstFakeObject */
 GST_START_TEST (test_fake_object_parentage)
 {
@@ -409,7 +419,8 @@ GST_START_TEST (test_fake_object_parentage)
   gst_object_unref (object2);
 }
 
-GST_END_TEST
+GST_END_TEST;
+
 /* parentage test dispose on GstFakeObject, since our testcase
  * does not handle the parent relation completely, the parent does
  * not hold a ref to the child, we cannot dispose the parent to
@@ -438,9 +449,12 @@ GST_START_TEST (test_fake_object_parentage_dispose)
   gst_object_unref (object2);
 }
 
-GST_END_TEST
+GST_END_TEST;
+
 /* test: try renaming a parented object, make sure it fails */
-    Suite * gst_object_suite (void)
+
+static Suite *
+gst_object_suite (void)
 {
   Suite *s = suite_create ("GstObject");
   TCase *tc_chain = tcase_create ("general");
@@ -460,8 +474,16 @@ GST_END_TEST
   tcase_add_test (tc_chain, test_fake_object_parentage_dispose);
   //tcase_add_checked_fixture (tc_chain, setup, teardown);
 
+  /* FIXME: GLib shouldn't crash here, but issue a warning and return a NULL
+   * object, or at least g_error() and then abort properly ... (tpm) */
+#ifdef HAVE_OSX
+  /* on OSX we get SIGBUS instead it seems */
+  tcase_add_test_raise_signal (tc_chain, test_fail_abstract_new, SIGBUS);
+#else
   /* SEGV tests go last so we can debug the others */
   tcase_add_test_raise_signal (tc_chain, test_fail_abstract_new, SIGSEGV);
+#endif
+
   return s;
 }
 
