@@ -122,7 +122,6 @@ print_query_types (const GstQueryType * types)
   }
 }
 
-#ifndef GST_DISABLE_ENUMTYPES
 #if 0
 static void
 print_event_masks (const GstEventMask * masks)
@@ -172,12 +171,6 @@ print_event_masks (const GstEventMask * masks)
   }
 }
 #endif
-#else
-static void
-print_event_masks (const GstEventMask * masks)
-{
-}
-#endif
 
 static char *
 get_rank_name (gint rank)
@@ -264,6 +257,40 @@ print_interfaces (GType type)
   }
 }
 
+static gchar *
+flags_to_string (GFlagsValue * vals, guint flags)
+{
+  GString *s = NULL;
+  guint flags_left, i;
+
+  /* first look for an exact match and count the number of values */
+  for (i = 0; vals[i].value_name != NULL; ++i) {
+    if (vals[i].value == flags)
+      return g_strdup (vals[i].value_nick);
+  }
+
+  s = g_string_new (NULL);
+
+  /* we assume the values are sorted from lowest to highest value */
+  flags_left = flags;
+  while (i > 0) {
+    --i;
+    if (vals[i].value != 0 && (flags_left & vals[i].value) == vals[i].value) {
+      if (s->len > 0)
+        g_string_append (s, " | ");
+      g_string_append (s, vals[i].value_nick);
+      flags_left -= vals[i].value;
+      if (flags_left == 0)
+        break;
+    }
+  }
+
+  if (s->len == 0)
+    g_string_assign (s, "(none)");
+
+  return g_string_free (s, FALSE);
+}
+
 static void
 print_element_properties_info (GstElement * element)
 {
@@ -309,8 +336,6 @@ print_element_properties_info (GstElement * element)
     if (param->flags & GST_PARAM_CONTROLLABLE) {
       if (!first_flag)
         g_print (", ");
-      else
-        first_flag = FALSE;
       g_print (_("controllable"));
     }
     n_print ("\n");
@@ -482,54 +507,30 @@ print_element_properties_info (GstElement * element)
           /* g_type_class_unref (ec); */
         } else if (G_IS_PARAM_SPEC_FLAGS (param)) {
           GParamSpecFlags *pflags = G_PARAM_SPEC_FLAGS (param);
-          GFlagsValue *values;
-          guint j = 0;
-          gint flags_value;
-          GString *cur_flags = NULL, *def_flags = NULL;
+          GFlagsValue *vals;
+          gchar *cur, *def;
 
-          values = G_FLAGS_CLASS (g_type_class_ref (param->value_type))->values;
-          flags_value = g_value_get_flags (&value);
+          vals = pflags->flags_class->values;
 
-          while (values[j].value_name) {
-            if (values[j].value & flags_value) {
-              if (cur_flags) {
-                g_string_append_printf (cur_flags, " | %s",
-                    values[j].value_nick);
-              } else {
-                cur_flags = g_string_new (values[j].value_nick);
-              }
-            }
-            if (values[j].value & pflags->default_value) {
-              if (def_flags) {
-                g_string_append_printf (def_flags, " | %s",
-                    values[j].value_nick);
-              } else {
-                def_flags = g_string_new (values[j].value_nick);
-              }
-            }
-            j++;
-          }
+          cur = flags_to_string (vals, g_value_get_flags (&value));
+          def = flags_to_string (vals, pflags->default_value);
 
           n_print
               ("%-23.23s Flags \"%s\" Default: 0x%08x, \"%s\" Current: 0x%08x, \"%s\"",
               "", g_type_name (G_VALUE_TYPE (&value)), pflags->default_value,
-              (def_flags ? def_flags->str : "(none)"), flags_value,
-              (cur_flags ? cur_flags->str : "(none)"));
+              def, g_value_get_flags (&value), cur);
 
-          j = 0;
-          while (values[j].value_name) {
+          while (vals[0].value_name) {
             g_print ("\n");
             if (_name)
               g_print ("%s", _name);
             g_print ("%-23.23s    (0x%08x): %-16s - %s", "",
-                values[j].value, values[j].value_nick, values[j].value_name);
-            j++;
+                vals[0].value, vals[0].value_nick, vals[0].value_name);
+            ++vals;
           }
 
-          if (cur_flags)
-            g_string_free (cur_flags, TRUE);
-          if (def_flags)
-            g_string_free (def_flags, TRUE);
+          g_free (cur);
+          g_free (def);
         } else if (G_IS_PARAM_SPEC_OBJECT (param)) {
           n_print ("%-23.23s Object of type \"%s\"", "",
               g_type_name (param->value_type));
@@ -704,7 +705,6 @@ print_clocking_info (GstElement * element)
   }
 }
 
-#ifndef GST_DISABLE_INDEX
 static void
 print_index_info (GstElement * element)
 {
@@ -717,12 +717,6 @@ print_index_info (GstElement * element)
     n_print ("Element has no indexing capabilities.\n");
   }
 }
-#else
-static void
-print_index_info (GstElement * element)
-{
-}
-#endif
 
 static void
 print_uri_handler_info (GstElement * element)
@@ -810,9 +804,9 @@ print_pad_info (GstElement * element)
       print_query_types (gst_pad_get_query_types (pad));
     }
 
-    if (pad->intlinkfunc != gst_pad_get_internal_links_default)
-      n_print ("      Has custom intconnfunc(): %s\n",
-          GST_DEBUG_FUNCPTR_NAME (pad->intlinkfunc));
+    if (pad->iterintlinkfunc != gst_pad_iterate_internal_links_default)
+      n_print ("      Has custom iterintlinkfunc(): %s\n",
+          GST_DEBUG_FUNCPTR_NAME (pad->iterintlinkfunc));
 
     if (pad->bufferallocfunc)
       n_print ("      Has bufferallocfunc(): %s\n",
@@ -982,18 +976,14 @@ print_element_list (gboolean print_all)
           g_print ("%s:  %s: %s\n", plugin->desc.name,
               GST_PLUGIN_FEATURE_NAME (factory),
               gst_element_factory_get_longname (factory));
-      }
-#ifndef GST_DISABLE_INDEX
-      else if (GST_IS_INDEX_FACTORY (feature)) {
+      } else if (GST_IS_INDEX_FACTORY (feature)) {
         GstIndexFactory *factory;
 
         factory = GST_INDEX_FACTORY (feature);
         if (!print_all)
           g_print ("%s:  %s: %s\n", plugin->desc.name,
               GST_PLUGIN_FEATURE_NAME (factory), factory->longdesc);
-      }
-#endif
-      else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
+      } else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
         GstTypeFindFactory *factory;
 
         factory = GST_TYPE_FIND_FACTORY (feature);
@@ -1037,6 +1027,80 @@ print_element_list (gboolean print_all)
   g_print ("\n");
 }
 
+static void
+print_all_uri_handlers (void)
+{
+  GList *plugins, *p, *features, *f;
+
+  plugins = gst_default_registry_get_plugin_list ();
+
+  for (p = plugins; p; p = p->next) {
+    GstPlugin *plugin = (GstPlugin *) (p->data);
+
+    features =
+        gst_registry_get_feature_list_by_plugin (gst_registry_get_default (),
+        plugin->desc.name);
+
+    for (f = features; f; f = f->next) {
+      GstPluginFeature *feature = GST_PLUGIN_FEATURE (f->data);
+
+      if (GST_IS_ELEMENT_FACTORY (feature)) {
+        GstElementFactory *factory;
+        GstElement *element;
+
+        factory = GST_ELEMENT_FACTORY (gst_plugin_feature_load (feature));
+        if (!factory) {
+          g_print ("element plugin %s couldn't be loaded\n", plugin->desc.name);
+          continue;
+        }
+
+        element = gst_element_factory_create (factory, NULL);
+        if (!element) {
+          g_print ("couldn't construct element for %s for some reason\n",
+              GST_OBJECT_NAME (factory));
+          gst_object_unref (factory);
+          continue;
+        }
+
+        if (GST_IS_URI_HANDLER (element)) {
+          const gchar *dir;
+          gchar **uri_protocols, *joined;
+
+          switch (gst_uri_handler_get_uri_type (GST_URI_HANDLER (element))) {
+            case GST_URI_SRC:
+              dir = "read";
+              break;
+            case GST_URI_SINK:
+              dir = "write";
+              break;
+            default:
+              dir = "unknown";
+              break;
+          }
+
+          uri_protocols =
+              gst_uri_handler_get_protocols (GST_URI_HANDLER (element));
+          joined = g_strjoinv (", ", uri_protocols);
+
+          g_print ("%s (%s, rank %u): %s\n", GST_OBJECT_NAME (factory), dir,
+              gst_plugin_feature_get_rank (GST_PLUGIN_FEATURE (factory)),
+              joined);
+
+          g_free (joined);
+          //g_strfreev (uri_protocols);
+        }
+
+        gst_object_unref (element);
+        gst_object_unref (factory);
+      }
+    }
+
+    gst_plugin_feature_list_free (features);
+  }
+
+  gst_plugin_list_free (plugins);
+}
+
 static void
 print_plugin_info (GstPlugin * plugin)
 {
@@ -1059,7 +1123,7 @@ print_plugin_features (GstPlugin * plugin)
   GList *features;
   gint num_features = 0;
   gint num_elements = 0;
-  gint num_types = 0;
+  gint num_typefinders = 0;
   gint num_indexes = 0;
   gint num_other = 0;
 
@@ -1079,17 +1143,13 @@ print_plugin_features (GstPlugin * plugin)
       n_print ("  %s: %s\n", GST_PLUGIN_FEATURE_NAME (factory),
           gst_element_factory_get_longname (factory));
       num_elements++;
-    }
-#ifndef GST_DISABLE_INDEX
-    else if (GST_IS_INDEX_FACTORY (feature)) {
+    } else if (GST_IS_INDEX_FACTORY (feature)) {
       GstIndexFactory *factory;
 
       factory = GST_INDEX_FACTORY (feature);
       n_print ("  %s: %s\n", GST_OBJECT_NAME (factory), factory->longdesc);
       num_indexes++;
-    }
-#endif
-    else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
+    } else if (GST_IS_TYPE_FIND_FACTORY (feature)) {
       GstTypeFindFactory *factory;
 
       factory = GST_TYPE_FIND_FACTORY (feature);
@@ -1107,7 +1167,7 @@ print_plugin_features (GstPlugin * plugin)
         g_print ("%s: %s: no extensions\n", plugin->desc.name,
             gst_plugin_feature_get_name (feature));
 
-      num_types++;
+      num_typefinders++;
     } else {
       n_print ("  %s (%s)\n", gst_object_get_name (GST_OBJECT (feature)),
           g_type_name (G_OBJECT_TYPE (feature)));
@@ -1120,8 +1180,8 @@ print_plugin_features (GstPlugin * plugin)
   n_print ("  %d features:\n", num_features);
   if (num_elements > 0)
     n_print ("  +-- %d elements\n", num_elements);
-  if (num_types > 0)
-    n_print ("  +-- %d types\n", num_types);
+  if (num_typefinders > 0)
+    n_print ("  +-- %d typefinders\n", num_typefinders);
   if (num_indexes > 0)
     n_print ("  +-- %d indexes\n", num_indexes);
   if (num_other > 0)
@@ -1136,14 +1196,12 @@ print_element_features (const gchar * element_name)
   GstPluginFeature *feature;
 
   /* FIXME implement other pretty print function for these */
-#ifndef GST_DISABLE_INDEX
   feature = gst_default_registry_find_feature (element_name,
       GST_TYPE_INDEX_FACTORY);
   if (feature) {
     n_print ("%s: an index\n", element_name);
     return 0;
   }
-#endif
   feature = gst_default_registry_find_feature (element_name,
       GST_TYPE_TYPE_FIND_FACTORY);
   if (feature) {
@@ -1342,7 +1400,10 @@ int
 main (int argc, char *argv[])
 {
   gboolean print_all = FALSE;
+  gboolean plugin_name = FALSE;
   gboolean print_aii = FALSE;
+  gboolean uri_handlers = FALSE;
+#ifndef GST_DISABLE_OPTION_PARSING
   GOptionEntry options[] = {
     {"print-all", 'a', 0, G_OPTION_ARG_NONE, &print_all,
         N_("Print all elements"), NULL},
@@ -1351,11 +1412,18 @@ main (int argc, char *argv[])
               "provides.\n                                       "
               "Useful in connection with external automatic plugin "
               "installation mechanisms"), NULL},
+    {"plugin", '\0', 0, G_OPTION_ARG_NONE, &plugin_name,
+        N_("List the plugin contents"), NULL},
+    {"uri-handlers", 'u', 0, G_OPTION_ARG_NONE, &uri_handlers,
+          N_
+          ("Print supported URI schemes, with the elements that implement them"),
+        NULL},
     GST_TOOLS_GOPTION_VERSION,
     {NULL}
   };
   GOptionContext *ctx;
   GError *err = NULL;
+#endif
 
 #ifdef ENABLE_NLS
   bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
@@ -1366,6 +1434,7 @@ main (int argc, char *argv[])
   if (!g_thread_supported ())
     g_thread_init (NULL);
 
+#ifndef GST_DISABLE_OPTION_PARSING
   ctx = g_option_context_new ("[ELEMENT-NAME | PLUGIN-NAME]");
   g_option_context_add_main_entries (ctx, options, GETTEXT_PACKAGE);
   g_option_context_add_group (ctx, gst_init_get_option_group ());
@@ -1374,31 +1443,46 @@ main (int argc, char *argv[])
     exit (1);
   }
   g_option_context_free (ctx);
+#else
+  gst_init (&argc, &argv);
+#endif
 
   gst_tools_print_version ("gst-inspect");
 
-  if (print_all && argc > 2) {
+  if (print_all && argc > 1) {
     g_print ("-a requires no extra arguments\n");
     return 1;
   }
 
+  if (uri_handlers && argc > 1) {
+    g_print ("-u requires no extra arguments\n");
+    exit (1);
+  }
+
   /* if no arguments, print out list of elements */
-  if (argc == 1 || print_all) {
+  if (uri_handlers) {
+    print_all_uri_handlers ();
+  } else if (argc == 1 || print_all) {
     print_element_list (print_all);
-    /* else we try to get a factory */
   } else {
+    /* else we try to get a factory */
     GstElementFactory *factory;
     GstPlugin *plugin;
     const char *arg = argv[argc - 1];
     int retval;
 
-    factory = gst_element_factory_find (arg);
-    /* if there's a factory, print out the info */
-    if (factory) {
-      retval = print_element_info (factory, print_all);
-      gst_object_unref (factory);
+    if (!plugin_name) {
+      factory = gst_element_factory_find (arg);
+
+      /* if there's a factory, print out the info */
+      if (factory) {
+        retval = print_element_info (factory, print_all);
+        gst_object_unref (factory);
+      } else {
+        retval = print_element_features (arg);
+      }
     } else {
-      retval = print_element_features (arg);
+      retval = -1;
     }
 
     /* otherwise check if it's a plugin */
